package org.kie.examples.osgi;

import org.kie.api.KieBase;
import org.kie.api.KieServices;
import org.kie.api.builder.ReleaseId;
import org.kie.api.runtime.KieSession;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;

/**
 * Sample OSGi BundleActivator illustrating KIE-CI usage.
 */
public class SampleKieOsgiActivator implements BundleActivator {

    // GAV of this project - will be installed into local Maven repo during build
    private static final ReleaseId VALID_RELEASE_ID = KieServices.Factory.get()
            .newReleaseId("org.kie.examples", "kie-ci-osgi-sample", "1.0.0-SNAPSHOT");

    // GAV of a project which is not in local Maven repo and therefore should be attempted to be downloaded by KIE-CI
    private static final ReleaseId NON_EXISTENT_RELEASE_ID = KieServices.Factory.get()
            .newReleaseId("org.kie.examples", "kie-ci-XXX-sample", "1.0.0-SNAPSHOT");

    private KieSession validReleaseIdKieSession;
    private KieSession invalidReleaseIdKieSession;

    public void start(final BundleContext bc) throws Exception {
        // this should succeed - accesses only local Maven repo
        this.validReleaseIdKieSession = this.createKieSession(VALID_RELEASE_ID);
        // this should fail on KieModule with given GAV does not exist, but throws CNF when connecting via HTTP
        this.invalidReleaseIdKieSession = this.createKieSession(NON_EXISTENT_RELEASE_ID);
    }

    private KieSession createKieSession(final ReleaseId releaseId) {
        final KieServices ks = KieServices.Factory.get();
        final KieBase kbase = ks.newKieContainer(releaseId).getKieBase();

        final KieSession kieSession = kbase.newKieSession();
        if (kieSession != null) {
            System.out.println("KieSession created - " + releaseId);
        } else {
            System.err.println("KieSession is null! - " + releaseId);
        }
        return kieSession;
    }

    public void stop(final BundleContext bc) throws Exception {
        this.disposeSession(this.validReleaseIdKieSession);
        this.disposeSession(this.invalidReleaseIdKieSession);
    }

    private void disposeSession(final KieSession kieSession) {
        if (kieSession!= null) {
            kieSession.dispose();
            System.out.println("KieSession disposed.");
        }
    }
}
